#!/usr/bin/env python

from behave import configuration
from behave import __main__

## add my wanted option to parser.
configuration.parser.add_argument('--browser', help="Specify browser to run, firefox or chrome")
configuration.parser.add_argument('--site', help="Specify site to run")

## command that run behave.
def main():
    return __main__.main()



# -*- coding: utf-8 -*-
from behave import *

from CustomOperators import *

from Page import *

from World import World as world
from Logger import Logger as log

import re
import datetime


"""
For create combination steps
"""

@step ("{execution}, accept fail")
def step_impl(context, execution):
    try:
        context.execute_steps(u"""When %s""" % execution)
    except: pass


@step ("{condition}, {execution}")
def step_impl(context, condition, execution):
    try:
        result = False
        context.execute_steps(u"""When %s""" % condition)
        result = True
    except: pass

    if result == True:
        context.execute_steps(u"""Then %s""" % execution)


# @step ("[remote] {Statement}")
# def step_impl(context, Statement):
#     world.driver = world.secondary_driver
#     context.execute_steps(u"""When %s""" % Statement)
#     world.driver = world.primary_driver


"""
Given step definitions
"""

@step ("user has [{page_name}/{path_name}] page opened") ##->given
def step_impl(context, page_name, path_name):
    page = world.find_page(page_name)
    page.go(path_name)
    page.wait_for_loading()


@step ("user has [{page_name}] page opened") ##->given
@step ("user has [{page_name}] page open")
@step ("user has [{page_name}] open")
def step_impl(context, page_name):
    page = world.find_page(page_name)
    page.go()
    page.wait_for_loading()


@step ("user browse to url '{url}'") ##->given
def step_impl(context, url):
    page = Page()
    page.url = url
    page.driver = world.driver
    page.go()
    page.wait_for_loading()


"""
When step definitions
"""

@step ("user enters '{value}' to the [{element_name}]") ##->when
@step ("user enters '{value}' to [{element_name}]")
def step_impl(context, element_name, value):
    element = world.find_element(element_name)
    if not element.send_keys(value):
        log.failed("Fail to enters value '%s' to '%s'" % (value, element.name))


@step ("user enters following lines to the [{element_name}]")
def step_impl(context, element_name):
    element = world.find_element(element_name)
    element.send_keys(context.text)


@step ("user enters date '{value}' to the [{element_name}]") ##->when
@step ("user enters date '{value}' to [{element_name}]")
def step_impl(context, value, element_name):
    inputdate = value
    today = datetime.date.today()

    if value.lower() == 'today':
        inputdate = "%s 00:00:00" % today
    elif value.lower() == 'tomorrow':
        tomorrow = today + datetime.timedelta(days=1)
        inputdate = "%s 00:00:00" % tomorrow

    element = world.find_element(element_name)
    element.send_keys_by_script(inputdate)


@step ("user enters date next '{value}' days to the [{element_name}]") ##->when
def step_impl(context, value, element_name):
    today = datetime.date.today()

    next_n_days = today + datetime.timedelta(days=int(value))
    inputdate = "%s 00:00:00" % next_n_days

    element = world.find_element(element_name)
    element.send_keys_by_script(inputdate)


@step ("user clears value on the [{element_name}]") ##->when
@step ("user clears input [{element_name}]")
def step_impl(context, element_name):
    element = world.find_element(element_name)
    element.send_keys_by_script("")


@step ("user selects the [{element_name}]") ##->when
@step ("user clicks the [{element_name}]") ##->when
@step ("user clicks [{element_name}] link")
@step ("user clicks [{element_name}] button")
def step_impl(context, element_name):
    element = world.find_element(element_name)
    if not element.click():
        log.failed("Fail to clicks element '%s'" % element.name)


@step ("user selects '{value}' on the [{element_name}]") ##->when
@step ("user selects '{value}' in [{element_name}]")
def step_impl(context, value, element_name):
    element = world.find_element(element_name)
    if not element.select(value):
        log.failed("Fail to selects value '%s' on '%s'" % (value, element.name))


@step ("user checks the [{element_name}]") ##->when
def step_impl(context, element_name):
    element = world.find_element(element_name)
    element.check()


@step ("user unchecks the [{element_name}]") ##->when
def step_impl(context, element_name):
    element = world.find_element(element_name)
    element.uncheck()


@step ("user moves mouse over the [{element_name}]") ##->when
@step ("user moves mouse over [{element_name}]")
def step_impl(context, element_name):
    element = world.find_element(element_name)
    element.move_mouse_over()


@step ("user uploads file '{file_path}' to the [{element_name}]") ##->when
@step ("user uploads file '{file_path}' to [{element_name}]")
def step_impl(context, file_path, element_name):
    element = world.find_element(element_name)
    element.send_keys_by_script(file_path)


@step ("user enters '{value}' to the popup")
def step_impl(context, value):
    alert = world.get_alert()
    alert.send_keys(value)


@step ("user accept the popup")
@step ('user clicks "OK" on the popup')
def step_impl(context):
    alert = world.get_alert_when_exist()

    if alert is not None:
        alert.accept()
    else:
        log.failed("Fail to accept popup alert")


@step ("user cancel the popup")
@step ('user clicks "Cancel" on the popup')
def step_impl(context):
    alert = world.get_alert_when_exist()

    if alert is not None:
        alert.dismiss()
    else:
        log.failed("Fail to cancel popup alert")


"""
Then step definitions
"""

@step ("the browser shows [{page_name}/{path_name}] page") ##->then
def step_impl(context, page_name, path_name):
    page = world.find_page(page_name)
    page.wait_for_loading(path_name)


@step ("the browser shows [{page_name}] page") ##->then
@step ("the system displays [{page_name}]")
def step_impl(context, page_name):
    page = world.find_page(page_name)
    page.wait_for_loading()


@step ("the page url is '{url}'") ##->then
@step ("the system url is '{url}'")
@step ("the system displays '{url}'")
def step_impl(context, url):
    page = Page(url=url)
    page.driver = world.driver
    page.wait_for_loading()

    if url == world.driver.current_url:
        return True
    else:
        log.failed("Verify url is?", world.driver.current_url, url)


@step ("the page url contains '{url}'") ##->then
@step ("the system url contains '{url}'")
def step_impl(context, url):
    page = Page(url=url)
    page.driver = world.driver
    page.wait_for_loading()

    if url in world.driver.current_url:
        return True
    else:
        log.failed("Verify url contains?", world.driver.current_url, url)


@step ("the [{element_name}] value is '{value}'") ##->then
@step ("the [{element_name}] shows '{value}'")
def step_impl(context, element_name, value):
    element = world.find_element(element_name)

    if element.value == value:
        return True
    else:
        log.failed("Verify value is?", element.value, value)


@step ("the [{element_name}] value is") ##->then
def step_impl(context, element_name):
    element = world.find_element(element_name)

    if element.value == context.text:
        return True
    else:
        log.failed("Verify value is? (multi-line)", element.value, context.text)


@step ("the [{element_name}] value is not '{value}'") ##->then
def step_impl(context, element_name, value):
    element = world.find_element(element_name)

    if element.value != value:
        return True
    else:
        log.failed("Verify value is not?", element.value, value)


@step ("the [{element_name}] value contains '{value}'") ##->then
@step ("the [{element_name}] contains '{value}'")
def step_impl(context, element_name, value):
    element = world.find_element(element_name)

    if value in element.value:
        return True
    else:
        log.failed("Verify value contains?", element.value, value)


@step ("the [{element_name}] value contains") ##->then
def step_impl(context, element_name):
    element = world.find_element(element_name)

    if context.text in element.value:
        return True
    else:
        log.failed("Verify value contains? (multi-line)", element.value, value)


@step ("the [{element_name}] value matches pattern '{pattern}'")
def step_impl(context, element_name, pattern):
    element = world.find_element(element_name)
    if re.match(pattern, element.value):
        return True
    else:
        log.failed("Verify value match with pattern?", element.value, pattern)


@step ("the [{element_name}] value is more than '{value}'") ##->then
@step ("the [{element_name}] more than '{value}'")
def step_impl(context, element_name, value):
    element = world.find_element(element_name)

    if element.value |more_than| value:
        return True
    else:
        log.failed(
            "Verify value is more than?",
            "value = %s" % element.value,
            "value > %s" % value
        )


@step ("the [{element_name}] value is more than or equal '{value}'") ##->then
@step ("the [{element_name}] more than or equal '{value}'")
def step_impl(context, element_name, value):
    element = world.find_element(element_name)

    if element.value |more_than_or_equal| value:
        return True
    else:
        log.failed(
            "Verify value is more than or equal?",
            "value = %s" % element.value,
            "value >= %s" % value
        )


@step ("the [{element_name}] value is less than '{value}'") ##->then
@step ("the [{element_name}] less than '{value}'")
def step_impl(context, element_name, value):
    element = world.find_element(element_name)

    if element.value |less_than| value:
        return True
    else:
        log.failed(
            "Verify value is less than?",
            "value = %s" % element.value,
            "value < %s" % value
        )


@step ("the [{element_name}] value is less than or equal '{value}'") ##->then
@step ("the [{element_name}] less than or equal '{value}'")
def step_impl(context, element_name, value):
    element = world.find_element(element_name)

    if element.value |less_than_or_equal| value:
        return True
    else:
        log.failed(
            "Verify value is less than or equal?",
            "value = %s" % element.value,
            "value <= %s" % value
        )


@step ("the [{element_name}] value is between '{value1}' and '{value2}'") ##->then
def step_impl(context, element_name, value1, value2):
    element = world.find_element(element_name)

    if (element.value |more_than_or_equal| value1) and (element.value |less_than_or_equal| value2):
        return True
    elif (element.value |less_than_or_equal| value1) and (element.value |more_than_or_equal| value2):
        return True
    else:
        log.failed(
            "Verify value is between?",
            "value = %s" % element.value,
            "value is between %s and %s" % (value1, value2)
        )


@step ("the [{element_name}] value is empty") ##->then
@step ("the [{element_name}] is blank")
def step_impl(context, element_name):
    element = world.find_element(element_name)

    if element.value == "":
        return True
    else:
        log.failed("Verify value is empty?", element.value, "<blank>")


@step ("the [{element_name}] value is not empty") ##->then
@step ("the [{element_name}] has a value")
def step_impl(context, element_name):
    element = world.find_element(element_name)

    if element.value != "":
        return True
    log.failed("Verify value is not empty?", element.value, "<any value>")


@step ("the [{element_name}] exists") ##->then
def step_impl(context, element_name):
    element = world.find_element(element_name)

    if element.wait_for_exist():
        return True
    else:
        log.failed("Verify element '%s' exists?" % element.name, "not exists", "exists")


@step ("the [{element_name}] does not exist") ##->then
def step_impl(context, element_name):
    element = world.find_element(element_name)

    if element.wait_for_not_exist():
        return True
    else:
        log.failed("Verify element '%s' exists?" % element.name, "not exists", "exists")


@step ("the [{element_name}] is visible") ##->then
def step_impl(context, element_name):
    element = world.find_element(element_name)

    if element.wait_for_attribute('visible', True):
        return True
    else:
        log.failed("Verify element '%s' visible?" % element.name, "not visible", "visible")


@step ("the [{element_name}] is invisible") ##->then
def step_impl(context, element_name):
    element = world.find_element(element_name)

    if element.wait_for_attribute('visible', False):
        return True
    else:
        log.failed("Verify element '%s' not visible?" % element.name, "visible", "not visible")


@step ("the [{element_name}] is enabled") ##->then
def step_impl(context, element_name):
    element = world.find_element(element_name)

    if element.wait_for_attribute('enabled', True):
        return True
    else:
        log.failed("Verify element '%s' enabled?" % element.name, "disabled", "enabled")


@step ("the [{element_name}] is disabled") ##->then
def step_impl(context, element_name):
    element = world.find_element(element_name)

    if element.wait_for_attribute('enabled', False):
        return True
    else:
        log.failed("Verify element '%s' disabled?" % element.name, "enabled", "disabled")


@step ("the [{element_name}] is selected") ##->then
@step ("the [{element_name}] is checked") ##->then
def step_impl(context, element_name):
    element = world.find_element(element_name)

    if element.wait_for_attribute('selected', True):
        return True
    else:
        log.failed("Verify element '%s' checked?" % element.name, "unchecked", "checked")


@step ("the [{element_name}] is not selected") ##->then
@step ("the [{element_name}] is unchecked") ##->then
def step_impl(context, element_name):
    element = world.find_element(element_name)

    if element.wait_for_attribute('selected', False):
        return True
    else:
        log.failed("Verify element '%s' unchecked?" % element.name, "checked", "unchecked")


@step ("the popup message shows '{value}'") ##->then
@step ("the popup shows '{value}'")
def step_impl(context, value):
    alert = world.get_alert_when_exist()

    if alert is None:
        log.failed("The popup alert not visible")

    if alert.text == value:
        return True
    else:
        log.failed("Verify popup message is?", alert.text, value)


@step ("the page has '{amount}' items of [{element_name}]") ##->then
@step ("the system displays '{amount}' items of [{element_name}]")
def step_impl(context, amount, element_name):
    element = world.find_element(element_name)

    if element.count == int(amount):
        return True
    else:
        log.failed(
            "Verify amount of '%s' is?" % element.name,
            "amount = %s" % element.count,
            "amount = %s" % amount
        )


@step ("the [{element_name}] tooltip text is '{value}'") ##->then
@step ("the [{element_name}] tooltip is '{value}'")
def step_impl(context, element_name, value):
    element = world.find_element(element_name)

    if element.get_attribute('title') == value:
        return True
    else:
        log.failed("Verify tooltip text is?", element.title, value)


@step ("the [{element_name}] class contains '{value}'")
def step_impl(context, element_name, value):
    element = world.find_element(element_name)
    classes = element.get_attribute('class')
    if value in classes:
        return True
    else:
        log.failed("Verify class contains?", classes, value)



# -*- coding: utf-8 -*-
import re

class Infix:
    def __init__(self, function):
        self.function = function
    def __ror__(self, other):
        return Infix(lambda x, self=self, other=other: self.function(other, x))
    def __or__(self, other):
        return self.function(other)
    def __rlshift__(self, other):
        return Infix(lambda x, self=self, other=other: self.function(other, x))
    def __rshift__(self, other):
        return self.function(other)
    def __call__(self, value1, value2):
        return self.function(value1, value2)

class NaturalOperator:
    """
    This class provide string compare operators in natural alphanumeric order.

    The original compare operators less than (<) and greather than (>) use computer alphanumeric
    while compare strings which is not the same behavior as human compare for example.

        print ( "5 USD" < "10 USD" ) >>> False

    So, this library defined new operators to solve this problem.

        print ( "5 USD" |less_than| "10 USD" ) >>> True
    """

    @classmethod
    def Sort(cls, unsortedList):
        """ Sort the list into natural alphanumeric order. """

        convert = lambda text: int(text) if text.isdigit() else text
        alphanum_key = lambda key: [ convert(c) for c in re.split('([0-9]+)', key) ]

        sortedList = list(unsortedList)
        sortedList.sort(key=alphanum_key)

        return sortedList

    @classmethod
    def LessThan(cls, value1, value2):
        """ Compare two strings with natural alphanumeric order. """

        unsortedList = []
        unsortedList.append(value1)
        unsortedList.append(value2)

        sortedList = cls.Sort(unsortedList)

        if unsortedList == sortedList and value1 != value2:
            return True
        else:
            return False

    @classmethod
    def MoreThan(cls, value1, value2):
        """ Compare two strings with natural alphanumeric order. """

        unsortedList = []
        unsortedList.append(value1)
        unsortedList.append(value2)

        sortedList = cls.Sort(unsortedList)
        sortedList.reverse()

        if unsortedList == sortedList and value1 != value2:
            return True
        else:
            return False

    @classmethod
    def Equal(cls, value1, value2):
        if value1 == value2:
            return True
        else:
            return False

    @classmethod
    def MoreThanOrEqual(cls, value1, value2):
        if cls.Equal(value1, value2) or cls.MoreThan(value1, value2):
            return True
        else:
            return False

    @classmethod
    def LessThanOrEqual(cls, value1, value2):
        if cls.Equal(value1, value2) or cls.LessThan(value1, value2):
            return True
        else:
            return False


##
## !!! declare natural sort operator !!!
##
natural_sort = NaturalOperator.Sort

##
## !!! declare compare operators !!!
##
equal = Infix(NaturalOperator.Equal)
more_than = Infix(NaturalOperator.MoreThan)
less_than = Infix(NaturalOperator.LessThan)
more_than_or_equal = Infix(NaturalOperator.MoreThanOrEqual)
less_than_or_equal = Infix(NaturalOperator.LessThanOrEqual)



# -*- coding: utf-8 -*-
import glob
import yaml

from yaml.parser import *
from yaml.scanner import *

from Page import *
from Element import *

from Logger import Logger as log

class PageFactory:

    @classmethod
    def check_yaml_syntax(cls, path):
        print "Checking yaml syntax..."

        error_files = 0

        for filename in glob.glob(path):
            content = open(filename, "r")
            docs = yaml.load_all(content)

            index = filename.rfind('/') + 1
            filename = filename[index:]

            try:
                for doc in docs: pass

            except ScannerError as exc:
                error_files += 1
                if hasattr(exc, 'problem_mark'):
                    mark = exc.problem_mark
                    print '  syntax error: file "%s", line %s' % (filename, mark.line)

            except ParserError as exc:
                error_files += 1
                if hasattr(exc, 'problem_mark'):
                    mark = exc.problem_mark
                    print '  syntax error: file "%s", line %s' % (filename, mark.line+1)

        if error_files == 0:
            print "  all file(s) passed."
            return True
        else:
            print "  found %s error file(s)." % error_files
            return False

    @classmethod
    def create_page(cls, doc):
        page = Page()
        page.name = doc['page']['name']

        if "url" in doc['page']:
            page.url = doc['page']['url']

        if "route" in doc['page']:
            page.route = doc['page']['route']

        if "url_paths" in doc['page']:
            for item in doc['page']['url_paths']:
                name = item['name']
                path = item['path']

                if not name in page.url_paths:
                    page.url_paths[name] = path
                else:
                    log.warning("Duplicate url path name!", {
                        'page': page.name,
                        'path': name
                    })

        if "elements" in doc:
            for item in doc['elements']:
                if not "name" in item:
                    log.warning("Found unnamed element!", {
                        'page': doc['page']['name']
                    })

                if not "xpath" in item:
                    log.warning("XPath not specified!", {
                        'page': doc['page']['name'],
                        'element': item['name']
                    })

                element = Element(item['name'], item['xpath'])
                element.driver = page.driver

                if not element.name in page.elements:
                    page.elements[element.name] = element
                else:
                    log.warning("Duplicate element name!", {
                        'page': page.name,
                        'element': element.name
                    })

        if "site_url" in doc:
            page.site_config = doc['site_url']
            page.site_config['default'] = page.url

        return page

    @classmethod
    def create_pages(cls, path, domain):
        pages = {}

        for filename in glob.glob(path):
            content = open(filename, "r")
            docs = yaml.load_all(content)

            for doc in docs:
                page = cls.create_page(doc)

                if page.url == "!!undefined":
                    page.url = cls.build_url(domain, page.route)

                if not page.name in pages:
                    pages[page.name] = page
                else:
                    log.warning("Duplicate page name!", { 'page': page.name })

        return pages

    @classmethod
    def get_domain(cls, filename, site):
        content = open(filename, "r")
        doc = yaml.load(content)

        if site in doc['sites']:
            return doc['sites'][site]
        else:
            log.failed("Not found site '%s' in config.yaml" % site)

    @classmethod
    def build_url(cls, domain, path):
        if domain == "": return domain
        if domain[-1] == '/': domain = domain[:-1]
        if path[0] == '/': path = path[1:]

        return "%s/%s" % (domain, path)



# -*- coding: utf-8 -*-
import sys
reload(sys); sys.setdefaultencoding('utf-8')

import glob
import yaml

from selenium import webdriver
from selenium.common.exceptions import *

from Logger import *
from Account import *
from PageObject import *


class Singleton:
    """
    A non-thread-safe helper class to ease implementing singletons.
    This should be used as a decorator -- not a metaclass -- to the
    class that should be a singleton.

    The decorated class can define one `__init__` function that
    takes only the `self` argument. Other than that, there are
    no restrictions that apply to the decorated class.

    To get the singleton instance, use the `Instance` method. Trying
    to use `__call__` will result in a `TypeError` being raised.

    Limitations: The decorated class cannot be inherited from.

    """

    def __init__(self, decorated):
        self._decorated = decorated

    def Instance(self):
        """
        Returns the singleton instance. Upon its first call, it creates a
        new instance of the decorated class and calls its `__init__` method.
        On all subsequent calls, the already created instance is returned.

        """
        try:
            return self._instance
        except AttributeError:
            self._instance = self._decorated()
            return self._instance

    def __call__(self):
        raise TypeError('Singletons must be accessed through `Instance()`.')

    def __instancecheck__(self, inst):
        return isinstance(inst, self._decorated)

@Singleton
class WorldContext:
    driver = None
    pages = {}
    accounts = {}

    current_page = None

    def OpenBrowser(self, browser="firefox"):
        if browser.lower() == 'firefox':
            self.driver = webdriver.Firefox()
        elif browser.lower() == 'chrome':
            self.driver = webdriver.Chrome()

        self.driver.implicitly_wait(15) ## set default wait time

    def CloseBrowser(self):
        for handle in self.driver.window_handles:
            self.driver.switch_to_window(handle)
            self.driver.close()

    def LoadPageObjects(self, path, site="default"):
        self.pages = {}

        for filename in glob.glob(path):
            content = open(filename, "r")
            docs = yaml.load_all(content)

            for doc in docs:
                page = Page(self.driver, doc, site)

                if not page.name in self.pages:
                    self.pages[page.name] = page
                else:
                    Log.Failed("Duplicate page name: '%s'" % page.name)

        return True

    def LoadAccounts(self, filename):
        self.accounts = {}

        content = open(filename, "r")
        doc = yaml.load(content)

        for acc in doc['accounts']:
            name = acc["name"]
            username = acc["username"]
            password = acc["password"]

            account = Account(name, username, password)
            self.accounts[name] = account

        return True


    def FindAccount(self, name):
        if name in self.accounts:
            return self.accounts[name]

        Log.Failed("Account not found in list: '%s'" % name)
        return None

    def FindPage(self, name):
        if name in self.pages:
            self.current_page = self.pages[name]
            return self.pages[name]

        Log.Failed("Page not found in list: '%s'" % name)
        self.current_page = None
        return None


    def FindElement(self, identifier):
        #element = self.CurrentPage.FindElement(identifier)
        element = self.current_page.__find_element(identifier)

        if element is not None:
            return element
        else:
            return Element(self.driver, identifier)


    def CreateDummyPage(self, url):
        pageobject = {}
        pageobject['page'] = {}
        pageobject['page']['name'] = 'noname'
        pageobject['page']['url'] = url

        page = Page(self.driver, pageobject)

        return page


    def VerifyURLIs(self, url):
        page = self.CreateDummyPage(url)
        page.WaitForPageLoaded()

        if url == self.driver.current_url:
            return True

        Log.Failed("Verify url is?", self.driver.current_url, url)
        return False


    def VerifyURLContains(self, url):
        page = self.CreateDummyPage(url)
        page.WaitForPageLoaded()

        if url in self.driver.current_url:
            return True

        Log.Failed("Verify url contains?", self.driver.current_url, url)
        return False


    @property
    def CurrentPage(self):
        return self.current_page

    def AcceptPopup(self):
        self.driver.switch_to_alert().accept()
        return True

    def DismissPopup(self):
        self.driver.switch_to_alert().dismiss()
        return True

    def SendKeysToPopup(self, value):
        self.driver.switch_to_alert().send_keys(value)
        return True

    def GetPopupMessage(self):
        popup = self.driver.switch_to_alert()
        return popup.text

    ## deprecated in v0.5.0
    def VerifyPopupMessage(self, message):
        popup = self.driver.switch_to_alert()
        if popup.text == message:
            return True

        Log.Failed("Verify popup message is?", popup.text, message)
        return False

World = WorldContext.Instance()



# -*- coding: utf-8 -*-

class Account:
    name = ""
    username = ""
    password = ""

    def __init__(self, name, username, password=""):
        self.name = name
        self.username = username
        self.password = password



# -*- coding: utf-8 -*-
import time

from selenium.webdriver.support.ui import Select
# from selenium.webdriver.support.ui import WebDriverWait
#
# from selenium.webdriver.common.by import By
# from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.action_chains import ActionChains


class Element:
    name = ""
    identifier = ""

    driver = None
    wait_timeout = 8

    def __init__(self, name="!!undefined", identifier="!!undefined"):
        self.name = name
        self.identifier = identifier
        self.wait_timeout = 8

    def get_instances(self):
        return self.driver.find_elements_by_xpath(self.identifier)

    def get_instance(self):
        instance = None

        if self.name == "!!undefined":
            if instance is None:
                try: instance = self.driver.find_element_by_id(self.identifier)
                except: pass

            if instance is None:
                try: instance = self.driver.find_element_by_name(self.identifier)
                except: pass

            if instance is None:
                try: instance = self.driver.find_element_by_class_name(self.identifier)
                except: pass

            if instance is None:
                try: instance = self.driver.find_element_by_link_text(self.identifier)
                except: pass
        else:
            try: instance = self.driver.find_element_by_xpath(self.identifier)
            except: pass

        return instance

    def get_instance_on_exist(self):
        self.wait_for_exist()
        return self.get_instance()

    def wait_for_exist(self):
        for n in range(0, self.wait_timeout * 2):
            element = self.get_instance()

            if element is not None: return True
            time.sleep(0.5)

        return False

    def wait_for_not_exist(self):
        for n in range(0, self.wait_timeout * 2):
            element = self.get_instance()

            if element is None: return True
            time.sleep(0.5)

        return False

    def wait_for_attribute(self, attr, expected):
        for n in range(0, self.wait_timeout * 2):
            element = self.get_instance()

            if element is None: continue

            if attr == 'enabled' and element.is_enabled() == expected:
                return True
            elif attr == 'visible' and element.is_displayed() == expected:
                return True
            elif attr == 'selected' and element.is_selected() == expected:
                return True
            elif element.get_attribute(attr) == expected:
                return True

            time.sleep(0.5)

        return False

    @property
    def count(self):
        return len(self.get_instances())

    @property
    def title(self):
        element = self.get_instance_on_exist()
        return element.get_attribute('title')

    @property
    def value(self):
        element = self.get_instance_on_exist()

        if element is None: return ""

        if element.tag_name in ['input', 'button', 'textarea']:
            return element.get_attribute('value')
        elif element.tag_name == 'select':
            return Select(element).first_selected_option.get_attribute('innerHTML')
        else:
            return element.get_attribute('innerHTML')

    @property
    def exists(self):
        element = self.get_instance_on_exist()
        if element is None:
            return False
        else:
            return True

    @property
    def enabled(self):
        element = self.get_instance_on_exist()
        if element is None:
            return False
        else:
            return element.is_enabled()

    @property
    def visible(self):
        element = self.get_instance_on_exist()
        if element is None:
            return False
        else:
            return element.is_displayed()

    @property
    def selected(self):
        element = self.get_instance_on_exist()
        if element is None:
            return False
        else:
            return element.is_selected()


    def get_attribute(self, attr):
        element = self.get_instance_on_exist()
        if element is None:
            return ""
        else:
            return element.get_attribute(attr)

    def send_keys(self, value):
        if not self.wait_for_exist():
            return False
        if not self.wait_for_attribute('visible', True):
            return False
        if not self.wait_for_attribute('enabled', True):
            return False

        element = self.get_instance()
        element.send_keys(value)

        return True

    def send_keys_by_script(self, value):
        element = self.get_instance_on_exist()

        if element is None:
            return False

        script = "arguments[0].value = '" + value + "'"
        self.driver.execute_script(script, element)
        return True


    def click(self):
        if not self.wait_for_exist():
            return False
        if not self.wait_for_attribute('visible', True):
            return False
        if not self.wait_for_attribute('enabled', True):
            return False

        element = self.get_instance()
        element.click()

        return True

    def click_by_script(self):
        element = self.get_instance_on_exist()

        if element is None:
            return False

        script = "arguments[0].click()"
        self.driver.execute_script(script, element)
        return True

    def select(self, value):
        if not self.wait_for_exist():
            return False
        if not self.wait_for_attribute('visible', True):
            return False
        if not self.wait_for_attribute('enabled', True):
            return False

        element = Select(self.get_instance())
        element.select_by_visible_text(value)

        return True

    def check(self):
        if not self.selected: self.click()
        return True

    def uncheck(self):
        if self.selected: self.click()
        return True

    def move_mouse_over(self):
        element = self.get_instance_on_exist()
        action = ActionChains(self.driver).move_to_element(element)
        action.perform()
        return True



# -*- coding: utf-8 -*-
import sys
import time

from urlparse import urlparse
from selenium.webdriver.support.ui import Select
from selenium.webdriver.common.action_chains import ActionChains

import selenium.webdriver.support.ui as ui

from Logger import Logger as log


class Page:
    driver = None

    name = ""
    route = ""

    url = ""
    url_paths = {}

    elements = {}

    #site = ""
    #site_config = {}

    def __init__(self, name="!!undefined", url="!!undefined"):
        self.name = name
        self.route = ""

        self.url = url
        self.url_paths = {}

        self.elements = {}

        #self.site = "default"
        #self.site_config = {}


    # def switch_site(self, site):
    #     if self.site == "default":
    #         return True
    #
    #     if self.site in self.site_config:
    #         self.url = self.site_config[self.site]
    #     else:
    #         log.warning("Not found site config '%s' in '%s'" % (self.site, self.name))


    def find_element(self, name):
        #names = name.split("|")
        #for name in names:
        if name in self.elements:
            element = self.elements[name]
            element.driver = self.driver

            return element
        else:
            return None


    def get_url(self, pathname=""):
        url = ""

        if pathname == "":
            url = self.url
        elif pathname in self.url_paths:
            domain = self.url
            path = self.url_paths[pathname]

            if domain[-1] == '/': domain = domain[:-1]
            if path[0] == '/': path = path[1:]

            url = "%s/%s" % (domain, path)
        else:
            log.warning("Not found url path '%s' in '%s'" % (pathname, self.name))

        return url


    def go(self, pathname=""):
        url = self.get_url(pathname)
        self.driver.get(url)
        return True


    def wait_for_loading(self, pathname="", timeout=15):
        self.driver.switch_to_window(self.driver.window_handles[-1])

        url = self.get_url(pathname)

        if "://" in url:
            uri = urlparse(url)
            url = uri.scheme + '://' + uri.hostname + uri.path

        ## wait until browser shows expected url
        wait = ui.WebDriverWait(self.driver, timeout)
        try:
            wait.until(lambda driver : self.driver.current_url.find(url) != -1)
            return True
        except:
            log.failed("Page load timeout (%s sec)." % timeout, self.driver.current_url, url)
            return False



#!/usr/bin/env python

import os
import sys
import textwrap

import subprocess

###
### command: cylon new ...
###
def new_project(directory):
    if os.path.exists(directory):
        print "Directory is already exists."
        return True
    else:
        os.makedirs(directory)
        os.makedirs("%s/pageobjects" % directory)
        os.makedirs("%s/features/steps" % directory)

        ## create config.yaml
        content = get_config_content()
        new_file(directory, "config.yaml", textwrap.dedent(content))

        ## create environment.py
        content = get_environment_content()
        new_file("%s/features" % directory, "environment.py", textwrap.dedent(content))

        ## create steps.py
        content = get_steps_content()
        new_file("%s/features/steps" % directory, "steps.py", textwrap.dedent(content))

        ## create runall.sh
        content = get_shell_runall_content()
        new_file(directory, "runall.sh", textwrap.dedent(content))

        ## create runall.bat
        content = get_batch_runall_content()
        new_file(directory, "runall.bat", textwrap.dedent(content))

        abs_path = os.path.abspath(directory)
        print  "New project created in: %s" % abs_path


def new_feature(filename):
    target = "./features"

    if not ".feature" in filename:
        filename = "%s.feature" % filename

    content = """
    @feature_tag
    Feature: Your feature description.

    @scenario_tag
    Scenario: Your scenario description.
    	Given your test preparation
    	When you do any action
    	Then you can verification
    """

    new_file(target, filename, textwrap.dedent(content))

def new_pageobject(filename):
    target = "./pageobjects"

    if not ".yaml" in filename:
        filename = "%s.yaml" % filename

    content = """
    --- ## %s
    page:
      name: %s
      url: http://yoursite.com

    elements:
    - name: title
      xpath: /html/head/title
    """ % (filename, filename.replace(".yaml", ""))

    new_file(target, filename, textwrap.dedent(content))

def new_file(target, filename, content=""):
    if not os.path.exists(target):
        print "Not found %s directory." % target
        return True

    if os.path.exists("%s/%s" % (target, filename)):
        print "File is already exists."
        return True

    new_file = open("%s/%s" % (target, filename), 'w')
    new_file.write(content)
    new_file.close()

    abs_path = os.path.abspath(target)
    print '%s created in: "%s".' % (filename, abs_path)

def update_file(target, filename, content=""):
    new_file = open("%s/%s" % (target, filename), 'w')
    new_file.write(content)
    new_file.close()

    abs_path = os.path.abspath(target)
    print '%s updated in: "%s".' % (filename, abs_path)

###
### command: cylon update ...
###
def update_project():
    ## update config.yaml
    if not os.path.exists("./config.yaml"):
        content = get_config_content()
        update_file("./", "config.yaml", textwrap.dedent(content))

    ## update environment.py
    content = get_environment_content()
    update_file("./features", "environment.py", textwrap.dedent(content))

###
### command: cylon run ...
###
def get_options_string(options):
    command = ""
    for option in options:
        option = "--" + option
        command = "%s %s" % (command, option)
    return command

def run_behook(options=""):
    return subprocess.call("behook --color --quiet --no-skipped %s" % options, shell=True)

# def run_all():
#     print "Running all features..."
#     os.system("behave --color --quiet --no-skipped")

# def run_tags(tags):
#     print "Running with tags: %s" % tags
#
#     argtags = ""
#     for tag in tags:
#         argtags = "%s --tags=%s" % (argtags, tag)
#
#     os.system("behave --color --quiet --no-skipped %s" % argtags.strip())

###
### print instruction
###
def print_instruction():
    content = get_instruction()
    print content

###
### get content functions
###
def get_config_content():
    content = """
    --- ## run configurations

    sites:
      ## at least we must have "default" site, dont't delete it ##
      default: http://www.yoursite.com

      ## add your sites domain here ##
      # develop: http://dev.yoursite.com

    ...
    """
    return content

# def get_accounts_content():
#     content = """
#     ### Define user accounts to use with login keyword ###
#     ---
#     accounts:
#
#     - name: pcms admin
#       username: test@domain.com
#       password: 123456
#     ...
#     """

def get_steps_content():
    content = """
    ###
    ### Implement additional step definitions here.
    ###
    """
    return content

def get_environment_content():
    content = """
    from project_cylon.Logger import *
    from project_cylon.PageFactory import *
    from project_cylon.CommonSteps import *

    from project_cylon.World import World as world

    def before_all(context):
        Logger.tracebacklimit(0)

        browser = "firefox"
        pageobject_files = "./pageobjects/*.yaml"

        site = "default"

        if hasattr(context.config, 'site') and context.config.site is not None:
            site = context.config.site

        domain = PageFactory.get_domain("./config.yaml", site)

        if PageFactory.check_yaml_syntax(pageobject_files) == True:
            world.pages = PageFactory.create_pages(pageobject_files, domain)
            world.open_browser(browser)
        else:
            Logger.failed("Stop running.")

    def after_all(context):
        world.close_browser()
    """
    return content

# def get_environment_content():
#     content = """
#     from project_cylon.Logger import *
#     from project_cylon.PageFactory import *
#     from project_cylon.CommonSteps import *
#
#     from project_cylon.World import World as world
#
#     def before_all(context):
#         browser = "firefox"
#
#         site_config = "default"
#         pageobject_files = "./pageobjects/*.yaml"
#
#         Logger.tracebacklimit(0)
#
#         if PageFactory.check_yaml_syntax(pageobject_files) == True:
#             world.pages = PageFactory.create_pages(pageobject_files, site_config)
#             world.open_browser(browser)
#         else:
#             Logger.failed("Stop running.")
#
#     def after_all(context):
#         world.close_browser()
#     """
#     return content

def get_shell_runall_content():
    content = """
    #!/bin/bash
    echo "Running all features..."
    behave --quiet --no-skipped
    """
    return content

# def get_shell_runtags_content():
#     content = """
#     #!/bin/bash
#
#     echo -n "Please specify tags to run >> "
#     read tags
#     echo "Running with tags: $tags"
#
#     behave --quiet --no-skipped --tags=$tags
#     """
#     return content

def get_batch_runall_content():
    content = """
    @echo off
    echo "Running all features..."
    behave --color --quiet --no-skipped
    pause
    """
    return content

# def get_batch_runtags_content():
#     content = """
#     """
#     return content

def get_instruction():
    content = """
    Usage:
    cylon <command> [options]

    Commands:
    new project <name>                Create new project directory.
    new feature <name>                Create new feature file.
    new pageobject <name>             Create new pageobject file.

    update project                    Update project files to compatible
                                      with current version.

    run all                           Run test with all feature.
    run [options]                     Run with specified options.

    Options:
    tags=<tags>                       Specified tags to run.
    site=<site>                       Specified site to run (see in config.yaml).
    """
    return content

###
### start script
###
def main():
    try:
        command = sys.argv[1].lower()

        if command == "new":
            sub = sys.argv[2].lower()
            filename = sys.argv[3]

            if sub == "project":
                new_project(filename)
            elif sub == "feature":
                new_feature(filename)
            elif sub in ["page", "pageobject"]:
                new_pageobject(filename)

        elif command == "run":
            sub = sys.argv[2].lower()

            if sub == "all":
                args = sys.argv[3:]
            else:
                args = sys.argv[2:]

            options = get_options_string(args)
            return run_behook(options)


        elif command == "update":
            sub = sys.argv[2].lower()

            if sub == "project":
                update_project()

    except: print_instruction()



# -*- coding: utf-8 -*-
import sys

class Logger:

    @classmethod
    def tracebacklimit(cls, value=1000):
        sys.tracebacklimit = int(value)

    @classmethod
    def warning(cls, message, fields={}):
        message = "WARN: %s\n" % message

        for key, value in fields.iteritems():
            message += "  %s: '%s'\n" % (key, value)

        print message
        raise RuntimeError

    @classmethod
    def failed(cls, message, actual="--", expect="--"):
        message = "FAIL: %s\n" % message

        if actual != "--" and expect != "--":
            message += "actual: '%s'\nexpect: '%s'\n" % (actual, expect)

        print message
        raise RuntimeError



from setuptools import setup

setup(name='project-cylon',
      version='1.1.10',
      description='Generic Web Acceptance Testing Framework',
      long_description=
      """
      Project-Cylon framework provide easy way to create web automation test without coding skill, just focus on the test scenario.
      """,
      url='https://github.com/gigapixel/project-cylon',
      author='Peerapat S., Kamol C. and Pongrapee J.',
      author_email='gigapixel7@gmail.com',
      license='MIT',
      classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Testing'
      ],
      packages=['project_cylon'],
      entry_points={
          'console_scripts': [
          'cylon=project_cylon.CLI:main', 'behook=project_cylon.behook:main'
      ]},
      install_requires=[
          'pyyaml',
          'behave',
          'selenium'
      ],
      zip_safe=False)



# -*- coding: utf-8 -*-
import sys
import time

from urlparse import urlparse
from selenium.webdriver.support.ui import Select
from selenium.webdriver.common.action_chains import ActionChains

import selenium.webdriver.support.ui as ui

from Logger import *


class Element:
    name = ""
    identifier = ""
    driver = None

    def __init__(self, driver, identifier, name='noname'):
        self.driver = driver
        self.name = name
        self.identifier = identifier

    def __find_by_xpath(self):
        try: return self.driver.find_element_by_xpath(self.identifier)
        except: return None

    def __find_by_id(self):
        try: return self.driver.find_element_by_id(self.identifier)
        except: return None

    def __find_by_name(self):
        try: return self.driver.find_element_by_name(self.identifier)
        except: return None

    def __find_by_class_name(self):
        try: return self.driver.find_element_by_class_name(self.identifier)
        except: return None

    def __find_by_link_text(self):
        try: return self.driver.find_element_by_link_text(self.identifier)
        except: return None

    def __get_instance(self):
        element = None
        self.driver.implicitly_wait(0) ## set wait time to 0 in case not found

        if self.name == "noname":
            element = self.__find_by_id()
            if element is None: element = self.__find_by_name()
            if element is None: element = self.__find_by_class_name()
            if element is None: element = self.__find_by_link_text()
        else:
            element = self.__find_by_xpath()

        self.driver.implicitly_wait(15) ## set wait time back
        return element

    # def __get_element_value(self, element):
    #     if element.tag_name in ['input', 'button', 'textarea']:
    #         return element.get_attribute('value')
    #     elif element.tag_name == 'select':
    #         return Select(element).first_selected_option.get_attribute('value')
    #     else:
    #         return element.get_attribute('innerHTML')

    def __get_value(self):
        element = self.GetInstance()

        if element.tag_name in ['input', 'button', 'textarea']:
            return element.get_attribute('value')
        elif element.tag_name == 'select':
            return Select(element).first_selected_option.get_attribute('value')
        else:
            return element.get_attribute('innerHTML')

    def GetInstance(self):
        element = self.__get_instance()

        if not element is None:
            return element
        else:
            Log.Failed("The element '%s' not found at identifier '%s'" % (self.name, self.identifier))

        # element = None
        #
        # ## set wait time to 0 in case not found
        # self.driver.implicitly_wait(0)
        #
        # if self.name == "noname":
        #     element = self.__find_by_id()
        #     if element is None: element = self.__find_by_name()
        #     if element is None: element = self.__find_by_class_name()
        #     if element is None: element = self.__find_by_link_text()
        # else:
        #     element = self.__find_by_xpath()
        #
        # ## set wait time back
        # self.driver.implicitly_wait(15)
        #
        # if logError and element is None:
        #     Log.Failed("The element '%s' not found at identifier '%s'" % (self.name, self.identifier))
        # return element


    def GetItems(self):
        elements = self.driver.find_elements_by_xpath(self.identifier)
        return elements

    @property
    def Name(self):
        return self.name

    @property
    def Identifier(self):
        return self.identifier

    @property
    def Exists(self):
        element = self.__get_instance()
        if not element is None:
            return True
        else:
            return False

    @property
    def Enabled(self):
        element = self.GetInstance()
        return element.is_enabled()

    @property
    def Visible(self):
        element = self.GetInstance()
        return element.is_displayed()

    @property
    def Selected(self):
        element = self.GetInstance()
        return element.is_selected()

    @property
    def Value(self):
        return self.__get_value()

    @property
    def TooltipText(self):
        element = self.GetInstance()
        return element.get_attribute('title')

    @property
    def Count(self):
        return len(self.GetItems())


    def SendKeys(self, value):
        element = self.GetInstance()
        element.send_keys(value)
        return True

    def SendKeysByScript(self, value):
        element = self.GetInstance()
        script = "arguments[0].value = '" + value + "'"
        self.driver.execute_script(script, element)
        return True

    def Click(self):
        element = self.GetInstance()
        element.click()
        return True

    def Select(self, value):
        element = Select(self.GetInstance())
        element.select_by_visible_text(value)
        return True

    def MouseOver(self):
        element = self.GetInstance()
        action = ActionChains(self.driver).move_to_element(element)
        action.perform()
        return True

    def Check(self):
        if not self.Selected:
            self.Click()
        return True

    def Uncheck(self):
        if self.Selected:
            self.Click()
        return True

    ###
    ### !!Verification method will raise error when failed, don't use for condition logic
    ### (deprecated in v0.5.0 and will be remove in v1.0.0)
    ###

    def VerifyValueIs(self, value):
        if self.Value == value:
            return True

        Log.Failed("Verify value is?", self.Value, value)
        return False

    def VerifyValueIsNot(self, value):
        if self.Value != value:
            return True

        Log.Failed("Verify value is not?", self.Value, value)
        return False

    def VerifyValueContains(self, value):
        if value in self.Value:
            return True

        Log.Failed("Verify value contains?", self.Value, value)
        return False

    def VerifyValueMoreThan(self, value):
        actual = self.Value
        expect = value

        if actual |more_than| expect:
            return True

        Log.Failed(
            "Verify value is more than?",
            "value = %s" % actual,
            "value > %s" % expect
        )
        return False

    def VerifyValueMoreThanOrEqual(self, value):
        actual = self.Value
        expect = value

        if actual |more_than_or_equal| expect:
            return True

        Log.Failed(
            "Verify value is more than or equal?",
            "value = %s" % actual,
            "value >= %s" % expect
        )
        return False

    def VerifyValueLessThan(self, value):
        actual = self.Value
        expect = value

        if actual |less_than| expect:
            return True

        Log.Failed(
            "Verify value is less than?",
            "value = %s" % actual,
            "value < %s" % expect
        )
        return False

    def VerifyValueLessThanOrEqual(self, value):
        actual = self.Value
        expect = value

        if actual |less_than_or_equal| expect:
            return True

        Log.Failed(
            "Verify value is less than or equal?",
            "value = %s" % actual,
            "value <= %s" % expect
        )
        return False

    def VerifyValueBetween(self, value1, value2):
        actual = self.Value

        if (actual |more_than_or_equal| value1) and (actual |less_than_or_equal| value2):
            return True
        if (actual |less_than_or_equal| value1) and (actual |more_than_or_equal| value2):
            return True

        Log.Failed(
            "Verify value is between?",
            "value = %s" % actual,
            "value is between %s and %s" % (value1, value2)
        )
        return False

    def VerifyValueIsBlank(self):
        if self.Value == '':
            return True

        Log.Failed("Verify value is empty?", self.Value, "<blank>")
        return False

    def VerifyHasValue(self):
        if not self.Value == '':
            return True

        Log.Failed("Verify value is not empty?", self.Value, "<any value>")
        return False

    def VerifyExists(self):
        if self.Exists:
            return True

        Log.Failed("Verify element '%s' exists?" % self.name, "not exists", "exists")
        return False

    def VerifyNotExists(self):
        if not self.Exists:
            return True

        Log.Failed("Verify element '%s' not exists?" % self.name, "exists", "not exists")
        return False

    def VerifyEnabled(self):
        if self.Enabled:
            return True

        Log.Failed("Verify element '%s' enabled?" % self.name, "disabled", "enabled")
        return False

    def VerifyDisabled(self):
        if not self.Enabled:
            return True

        Log.Failed("Verify element '%s' disabled?" % self.name, "enabled", "disabled")
        return False

    def VerifyVisible(self):
        if self.Visible:
            return True

        Log.Failed("Verify element '%s' visible?" % self.name, "not visible", "visible")
        return False

    def VerifyNotVisible(self):
        if not self.Visible:
            return True

        Log.Failed("Verify element '%s' not visible?" % self.name, "visible", "not visible")
        return False

    def VerifyIsChecked(self):
        if self.Selected:
            return True

        Log.Failed("Verify element '%s' checked?" % self.name, "unchecked", "checked")
        return False

    def VerifyIsUnchecked(self):
        if not self.Selected:
            return True

        Log.Failed("Verify element '%s' unchecked?" % self.name, "checked", "unchecked")
        return False

    def VerifyItemsCount(self, count):
        if self.Count == int(count):
            return True

        Log.Failed(
            "Verify amount of '%s' is?" % self.name,
            "amount = %s" % self.Count,
            "amount = %s" % count
        )
        return False

    def VerifyTooltip(self, value):
        if self.Tooltip == value:
            return True

        Log.Failed("Verify tooltip text is?", self.Tooltip, value)
        return False

    # def VerifyValueSortedAscending(self):
    #     unsorted_values = []
    #     elements = self.GetItems()
    #
    #     for element in elements:
    #         unsorted_values.append(self.__get_element_value(element))
    #
    #     sorted_values = natural_sort(unsorted_values)
    #
    #     if unsorted_values == sorted_values:
    #         return True
    #
    #     Log.Failed(
    #         "Verify value sorted ascending?",
    #         "[%s]" % ','.join(unsorted_values),
    #         "[%s]" % ','.join(sorted_values)
    #     )
    #     return False

    # def VerifyValueSortedDescending(self):
    #     return False


class Page:
    name = ""
    url = ""
    url_paths = {}
    driver = None

    elements = {}

    def __init__(self, driver, pageobject, site="default"):
        self.driver = driver
        self.name = pageobject['page']['name']
        self.url = pageobject['page']['url']

        if "url_paths" in pageobject['page']:
            for item in pageobject['page']['url_paths']:
                name = item['name']
                path = item['path']

                if not name in self.url_paths:
                    self.url_paths[name] = path
                else:
                    Log.Warning("%s\nDuplicate url path name: '%s'" % (self.name, name))

        ## load elements
        self.elements = {}

        if "elements" in pageobject:
            for item in pageobject['elements']:
                element = Element(self.driver, item['xpath'], item['name'])

                if not element.name in self.elements:
                    self.elements[element.name] = element
                else:
                    Log.Warning("%s\nDuplicate element name: '%s'" % (self.name, element.name))

        if "site_url" in pageobject and site != "default":
            if site in pageobject['site_url']:
                self.url = pageobject['site_url'][site]
            else:
                Log.Warning("Not found site url '%s' in '%s'" % (site, self.name))


    def __find_element(self, name):
        names = name.split("|")
        for name in names:
            if name in self.elements:
                return self.elements[name]
        return None


    def GetURL(self, pathname=""):
        url = ""

        if pathname == "":
            url = self.url
        elif pathname in self.url_paths:
            domain = self.url
            path = self.url_paths[pathname]

            if domain[-1] == '/': domain = domain[:-1]
            if path[0] == '/': path = path[1:]

            url = domain + '/' + path
        else: Log.Warning("Not found url path '%s' in '%s'" % (pathname, self.name))

        return url


    def Go(self, pathname=""):
        url = self.GetURL(pathname)
        self.driver.get(url)
        return True

    def FindElement(self, name):
        element = self.__find_element(name)
        if element is not None:
            return element
        else:
            Log.Failed("Element not found in list: '%s'" % name)
        # names = name.split("|")
        # for name in names:
        #     if name in self.elements:
        #         return self.elements[name]
        # return None

    def VerifyPage(self, pathname=""):
        return self.WaitForPageLoaded(pathname)

    def WaitForPageLoaded(self, pathname="", timeout=15):
        self.driver.switch_to_window(self.driver.window_handles[-1])

        url = self.GetURL(pathname)

        if "://" in url:
            uri = urlparse(url)
            url = uri.scheme + '://' + uri.hostname + uri.path

        ## wait until browser shows expected url
        wait = ui.WebDriverWait(self.driver, timeout)
        try:
            wait.until(lambda driver : self.driver.current_url.find(url) != -1)
            return True
        except:
            Log.Failed("Page load timeout (%s sec)." % timeout, self.driver.current_url, url)
            return False



class Widget:
    driver = None

    name = ""
    elements = {}

    def __init__(self):
        self.driver = None
        self.name = ""
        self.elements = {}

    def find_element(self, name):
        if name in self.elements:
            element = self.elements[name]
            element.driver = self.driver

            return element
        else:
            return None



# -*- coding: utf-8 -*-
from selenium import webdriver

from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

from Element import *
from Logger import Logger as log


class World:

    driver = None
    pages = {}
    current_page = None

    @classmethod
    def open_browser(cls, browser="firefox"):
        if browser is "firefox":
            cls.driver = webdriver.Firefox()
        elif browser is "chrome":
            cls.driver = webdriver.Chrome()
        else:
            log.failed("The %s browser is not supported." % browser)

    @classmethod
    def close_browser(cls):
        if cls.driver is not None:
            for handle in cls.driver.window_handles:
                cls.driver.switch_to_window(handle)
                cls.driver.close()

    @classmethod
    def find_page(cls, name):
        if name in cls.pages:
            page = cls.pages[name]
            page.driver = cls.driver

            cls.current_page = page
            return page
        else:
            log.failed("Page not found in list: '%s'" % name)
            cls.current_page = None
            return None

    @classmethod
    def find_page_by_url(cls, url):
        for page in cls.pages:
            if url in page.url or page.url in url:
                page.driver = cls.driver
                cls.current_page = page

                log.warning("Current page was changed to: '%s'" % page.name)
                return page

        log.warning("Not found page match with url '%s'" % url)
        cls.current_page = None
        return None


    @classmethod
    def find_element(cls, identifier):
        element = cls.current_page.find_element(identifier)

        if element is not None:
            return element
        else:
            element = Element(identifier=identifier)
            element.driver = cls.driver
            return element

    @classmethod
    def get_alert(cls):
        return cls.driver.switch_to_alert()

    @classmethod
    def get_alert_when_exist(cls):
        wait = WebDriverWait(cls.driver, 10)

        try:
            wait.until(EC.alert_is_present())
            alert = cls.driver.switch_to_alert()
            return alert
        except:
            return None
